disp(' ')
disp(' rainflow_sine.m   ver 1.0   August 5, 2014')
disp(' by Tom Irvine   Email: tom@vibrationdata.com')
disp(' ')
disp(' This program calculates the response of a single-degree-of-freedom')
disp(' system to a sine or sine sweep base input time history. ')
disp(' ')
disp(' It also counts the rainflow cycles and calculates the relative ');
disp(' fatigue damage. ');
disp(' ')
disp(' The input time history must have two columns: time(sec) & accel')
disp(' ')
%
close all;
%
clear omega;
clear omegad;
clear dt;
clear sr;
clear damp;
clear a1;
clear a2;
clear b1;
clear b2;
clear c1;
clear c2;
clear sum;
clear t;
clear tt;
clear y;
clear yy;
clear THM;
clear x_pos;
clear x_neg;
clear rd_pos;
clear rd_neg;
clear a_resp;
clear x_resp;
clear aa_forward;
clear rv_forward;
clear rd_forward;
clear back;
clear length;
clear acceleration;
clear amp_cycles
%
fig_num=1;
%
iu=1;
%
[t,y,dt,sr,tmx,tmi,n,ncontinue]=enter_time_history();
%
%
figure(fig_num);
fig_num=fig_num+1;
plot(t,y);
grid on;
title('Base Input Acceleration Time History ');
xlabel('Time (sec) ');
ylabel('Accel (G)');
%
%
if(ncontinue==1)
%
disp(' ')
fn(1)=input(' Enter the natural frequency (Hz)  ');
%
disp(' ')
idamp=input(' Enter damping format:  1= damping ratio   2= Q  ');	
%
disp(' ')
if(idamp==1)
    damp=input(' Enter damping ratio (typically 0.05) ');
else
    Q=input(' Enter the amplification factor (typically Q=10) ');
    damp=1./(2.*Q);
end
%
disp(' ');
bex=input(' Enter the fatigue exponent ');
%
%  Initialize coefficients
%
[a1,a2,b1,b2,b3,rd_a1,rd_a2,rd_b1,rd_b2,rd_b3]=...
                                   srs_coefficients(fn,damp,dt); 
%   
yy=y;
%
%  SRS engine
%
for j=1:1
%
    back   =[     1, -a1(j), -a2(j) ];
%
    disp(' ')
    disp(' Calculating absolute acceleration');
%
    aa_forward=[ b1(j),  b2(j),  b3(j) ];
%    
    a_resp=filter(aa_forward,back,yy);
%
    x_pos(j)= max(a_resp);
    x_neg(j)= min(a_resp);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
    k=1; 
    disp(' Begin slope calculation ')
%
    y=a_resp;
    m=length(y)-1;
    slope1=(  y(2)-y(1));
    for i=2:m
        slope2=(y(i+1)-y(i));
        if((slope1*slope2)<=0)
          a(k)=y(i);
          t(k)=i;
          k=k+1;
        end
        slope1=slope2;
    end
%
    a(k)=y(m+1);
    t(k)=t(k-1)+1;
    k=k+1;
%
    disp(' End slope calculation ')
%
    clear temp;
    temp(1:k-1)=a(1:k-1);
    clear a;
    a=temp;
%
    clear temp;
    temp(1:k-1)=t(1:k-1);
    clear t;
    t=temp;
%
    clear aa;
    sza=size(a);
    if(sza(2)>sza(1))
        a=a';
    end
    szt=size(t);
    if(szt(2)>szt(1))
        t=t';
    end
%
    aa=[t a];
%
    cc=a;
%
    p=length(a);
%
    amp_cycles=zeros(p-1,2);
%
    D=0;
    for i=2:p
        amp=0.5*abs(a(i)-a(i-1));
        D=D+0.5*amp^bex;
%
       amp_cycles(i-1,:)=[amp 0.5];
%
    end    
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
end
%
disp(' ')
disp(' Acceleration Response ')
disp(' ')
mu=mean(a_resp);
sd=std(a_resp);
arms=sqrt(sd^2+mu^2);
peak=max(abs(a_resp));
if(abs(x_neg(1))>peak)
    peak=abs(x_neg(1));
end
%
clear aar;
aar=a_resp-mu;
aar=aar.^4;
kurtosis=sum(aar)/(n*sd^4);
%
T=(tmx-tmi);
ccc=sqrt(2*log(fn(1)*T));
ray=ccc+0.5772/ccc;
disp(' Rayleigh distibution expected peak for broadband random input ');
out10=sprintf('                 = %7.2f sigma',ray);
out11=sprintf('                 = %7.2f G ',ray*sd);
disp(out10);
disp(out11);
%
 out9=sprintf('\n  absolute peak = %8.2f G',peak);
out10=sprintf('\n        maximum = %9.2f G',x_pos(1));
  out11=sprintf('        minimum = %9.2f G',x_neg(1));
  out12=sprintf('        overall = %9.2f GRMS',arms);
out13=sprintf('\n           mean = %9.2f G',mu);
  out14=sprintf('        std dev = %9.2f G',sd);
  out15=sprintf('   crest factor = %9.2f',peak/sd);
  out16=sprintf('       kurtosis = %9.2f\n',kurtosis);  
%
  out17=sprintf(' fatigue damage = %8.4g ',D);
%
disp(out9);
disp(out10)
disp(out11)
disp(out12)
disp(out13)
disp(out14)
disp(out15)
disp(out16)
disp(out17)
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  
nn=length(a_resp);
%
tmin=t(1);
tmax=(t(1)+(nn-1)*dt);
%
tt=linspace(tmin,tmax,nn);
ymin=1.3*min(a_resp);
ymax=1.3*max(a_resp);
if(abs(ymin)>abs(ymax))
    ymax=abs(ymin); 
end
if(abs(ymax)>abs(ymin))
    ymin=-abs(ymax); 
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    
%  Output 
%
disp(' ');
acceleration(:,1)=tt';
acceleration(:,2)=a_resp';
%
disp(' ')
disp('Output Acceleration Response Text File?')
choice=input(' 1=yes  2=no  ' );
disp(' ')
%
if choice == 1 
%
      [writefname, writepname] = uiputfile('*.dat','Save acceleration data as');
	  writepfname = fullfile(writepname, writefname);
%      
	  fid = fopen(writepfname,'w');
      clear length;
      n=length(tt);
%
      for i=1:n
	    fprintf(fid,' %g \t %g\n',tt(i),a_resp(i));
      end
%
      fclose(fid);
%    
end
%
%  Plot acceleration response
%
Q=1./(2.*damp);
t_string = sprintf(' SDOF Acceleration Response   fn=%g Hz   Q=%g ',fn(1),Q);
y_label=sprintf('Accel (G)');
x_label=sprintf('Time (sec)');   
[fig_num]=plot_TH(fig_num,x_label,y_label,t_string,acceleration);
%
%
disp(' ');
disp(' Output arrays: ');
disp(' ');
disp('          Acceleration response:  acceleration - time(sec) accel(G)');
disp('                Rainflow cycles:  amp_cycles   - amplitude & cycles');
%
end